//  "myGL.cpp"
//   "myGL"


#include "myGL.h"
#include "sazerlend.h"
#include <math.h>

#define DRAW_PARTS 30


myGL::myGL(QObject *p)
{
        scale = 1.0;
        c_x = 0.0;
        c_y = 0.0;

	drawCoordinateSystem = true;

	setMouseTracking(true);
}

void myGL::setSazerlend(sazerlend *s)
{
	SAZ = s;
}

void myGL::slot_algorythm(void)
{
    SAZ -> startAlgorythm();
    updateGL();
}

void myGL::zoomScene(int delta)
{
	float x, y;
	x = (c_x) / scale;
	y = (c_y) / scale;
	scale = delta;
	c_x = x * scale;
	c_y = y * scale;
	resizeGL(this -> width(), this -> height());
	updateGL();
}


//================================================================================================================
//=================================================================================================
//================================================================================================================
void myGL::draw()
{
	if(drawCoordinateSystem)
	{
/*		glColor4f(0.3, 0.3, 0.3, 0.5);
		glBegin(GL_LINES);
			glVertex2d(- c_x - width() / 2., 0);
			glVertex2d(width() / 2. - c_x, 0);
		glEnd();*/
	}
	SAZ -> draw();
}

void myGL::drawPoint(int x, int y)
{
	glColor3f(0.0, 0.0, 0.0);
	glPointSize(1.0);
	glBegin(GL_POINTS);
                glVertex2d(x * scale, y * scale);
        glEnd();
}

void myGL::drawLine(int x1, int y1, int x2, int y2)
{
        glColor3f(0.75, 0.0, 0.0);
        glLineWidth(1.5);
        glBegin(GL_LINES);
                glVertex2d(x1 * scale, y1 * scale);
                glVertex2d(x2 * scale, y2 * scale);
	glEnd();	
}

void myGL::drawRect(int x1, int y1, int x2, int y2)
{
    glColor3f(0.0, 0.5, 0.0);
    glLineWidth(0.75);
    glBegin(GL_LINES);
        glVertex2d(x1 * scale - 100, y1 * scale);
        glVertex2d(x2 * scale + 100, y1 * scale);
        glVertex2d(x1 * scale - 100, y2 * scale);
        glVertex2d(x2 * scale + 100, y2 * scale);
        glVertex2d(x1 * scale, y1 * scale - 100);
        glVertex2d(x1 * scale, y2 * scale + 100);
        glVertex2d(x2 * scale, y1 * scale - 100);
        glVertex2d(x2 * scale, y2 * scale + 100);
    glEnd();

    glColor3f(0.0, 0.3, 0.0);
    glLineWidth(2.0);
    glBegin(GL_LINE_LOOP);
        glVertex2d(x1 * scale, y1 * scale);
        glVertex2d(x2 * scale, y1 * scale);
        glVertex2d(x2 * scale, y2 * scale);
        glVertex2d(x1 * scale, y2 * scale);
    glEnd();
}


//================================================================================================================
//=========================================OpenGL==========================================================
//================================================================================================================
void myGL::initializeGL()
{
	qglClearColor(Qt::white);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);
}

void myGL::resizeGL(int width, int height)
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(-width/2.0 + c_x, width/2.0 + c_x, -height/2.0 + c_y, height/2.0 + c_y);
	glMatrixMode(GL_MODELVIEW);
	updateGL();
}

void myGL::showEvent(QShowEvent *event)
{
	updateGL();
}

void myGL::paintGL()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	draw();
}

void myGL::timerEvent(QTimerEvent *event)
{
	updateGL();
}


//================================================================================================================
//====================================================================================================
//================================================================================================================
void myGL::mousePressEvent(QMouseEvent *event)
{
        if(event -> buttons() & Qt::RightButton)
	{
                xx = (int)trX(event -> x());
                yy = (int)trY(event -> y());
		SAZ -> setRect(xx, yy, xx, yy);
	}
        else if(event -> buttons() & Qt::MidButton)
	{
		attachX = event -> x();
		attachY = event -> y();
	}
        else if(event -> buttons() & Qt::LeftButton)
        {
                xx = (int)trX(event -> x());
                yy = (int)trY(event -> y());
                SAZ -> setLine(xx, yy, xx, yy);
        }
}

void myGL::mouseReleaseEvent(QMouseEvent *event)
{
        if(event -> buttons() & Qt::RightButton)
                SAZ -> setRect(xx, yy, (int)trX(event -> x()), (int)trY(event -> y()));
        else if(event -> buttons() & Qt::LeftButton)
                SAZ -> setLine(xx, yy, (int)trX(event -> x()), (int)trY(event -> y()));
}

void myGL::mouseMoveEvent(QMouseEvent *event)
{
        if(event -> buttons() & Qt::RightButton)
	{
                SAZ -> setRect(xx, yy, (int)trX(event -> x()), (int)trY(event -> y()));
		updateGL();
	}
        else if(event -> buttons() & Qt::MidButton)
	{
		double nax, nay;
		nax = event -> x();
		nay = event -> y();
		c_x += attachX - nax;
		c_y -= attachY - nay;
		attachX = nax;
		attachY = nay;
		resizeGL(this -> width(), this -> height());
	}
        else if(event -> buttons() & Qt::LeftButton)
        {
                SAZ -> setLine(xx, yy, (int)trX(event -> x()), (int)trY(event -> y()));
                updateGL();
        }
}

void myGL::mouseDoubleClickEvent(QMouseEvent *event)
{

}

void myGL::wheelEvent(QWheelEvent *event)
{
	float zoom = scale + event -> delta() * 0.01 * pow(1.03, scale);
        if(zoom < 1)
                zoom = 1;
	if(zoom > 50)
		zoom = 50;
	zoomScene(zoom);
}

//================================================================================================================
//======================================= =================================================
//================================================================================================================
double myGL::trX(double x)
{
	return (c_x - this -> width() / 2 + x) / scale;
}

double myGL::trY(double y)
{
	return (c_y + this -> height() / 2 - y) / scale;
}

double myGL::trBackX(double x)
{
	return x * scale + this -> width() / 2. - c_x;
}

double myGL::trBackY(double y)
{
	return - y * scale + this -> height() / 2. + c_y;
}
